@@ -19,19 +19,18 @@
 #define _MAIN_WIDGET_H_
 
 #include "config.h"
-#include <qframe.h>
-#include <qptrlist.h>
-#include <qwidget.h>
 
-#include "kwave/SignalWidget.h"
+#include <QFrame>
+#include <QList>
+#include <QWidget>
+
+#include "libgui/SignalWidget.h"
 
-class QAccel;
 class QComboBox;
 class QScrollBar;
 
-class KButtonBox;
 class KStatusBar;
-class KURL;
+class KUrl;
 
 class MenuManager;
 class MultiStateWidget;
@@ -49,7 +48,7 @@ public:
      * Constructor.
      * @param parent parent widget
      */
-    MainWidget (QWidget *parent);
+    MainWidget(QWidget *parent);
 
     /**
      * Returns true if this instance was successfully initialized, or
@@ -66,7 +65,7 @@ public:
      * @param url URL of the file to be loaded
      * @return 0 if succeeded or error code < 0
      */
-    int loadFile(const KURL &url);
+    int loadFile(const KUrl &url);
 
     /**
      * Closes the current signal.
@@ -125,8 +124,6 @@ public slots:
 
     void forwardCommand(const QString &command);
 
-    void resetChannels();
-
     void parseKey(int key);
 
     /** calls setZoom() of the signal widget */
@@ -151,25 +148,6 @@ public slots:
 
 private slots:
 
-    /**
-     * Forwards the zoomChanged signal of the iternal view window
-     * by emitting sigZoomChanged.
-     */
-    void forwardZoomChanged(double zoom);
-
-    /**
-     * Forwards the selectedTimeInfo signal of the internal view window
-     * by emitting selectedTimeInfo.
-     */
-    void forwardSelectedTimeInfo(unsigned int offset, unsigned int length,
-                                 double rate);
-
-    /**
-     * Forwards the sigMouseChanged signal of the internal view window
-     * by emitting sigMouseChanged again.
-     */
-    void forwardMouseChanged(int mode);
-
     /**
      * Called if a track has been added. Updates the display by
      * resizing/re-positioning the channel controls and the signal
@@ -190,12 +168,21 @@ private slots:
      */
     void slotTrackDeleted(unsigned int track);
 
+    /** updates the scrollbar */
+    void updateViewInfo(unsigned int, unsigned int, unsigned int);
+
     /**
      * Connected to the vertical scrollbar and called if the value
      * has changed so that the signal display and the channel
      * controls have to be moved.
      */
-    void scrollbarMoved(int newval);
+    void verticalScrollBarMoved(int newval);
+
+    /** refresh the scale and position of the horizontal scrollbar */
+    void refreshHorizontalScrollBar();
+
+    /** Connected to the horizontal scrollbar for scrolling left/right */
+    void horizontalScrollBarMoved(int newval);
 
 signals:
 
@@ -226,9 +213,8 @@ signals:
 
 private:
 
-    QAccel *keys;
-
-    OverViewWidget *m_slider;
+    /** overview widget */
+    OverViewWidget *m_overview;
 
     /** QFrame that contains the signal widget. */
     QFrame m_signal_frame;
@@ -236,16 +222,16 @@ private:
     /** the widget that shows the signal */
     SignalWidget m_signal_widget;
 
-    QFrame *frmChannelControls;
+    QFrame *m_frm_channel_controls;
 
     /** vertical scrollbar, only visible if channels do not fit vertically */
-    QScrollBar *m_scrollbar;
+    QScrollBar *m_vertical_scrollbar;
+
+    /** horizontal scrollbar, always visible */
+    QScrollBar *m_horizontal_scrollbar;
 
     /** array of lamps, one for each channel */
-    QPtrList<MultiStateWidget> m_lamps;
-
-//    /** array of speaker icons, one for each channel */
-//    QPtrList<MultiStateWidget> m_speakers;
+    QList<MultiStateWidget *> m_lamps;
 
     /** the last number of channels (for detecting changes) */
     unsigned int m_last_tracks;
