@@ -19,10 +19,11 @@
 #define _MENU_ROOT_H_
 
 #include "config.h"
-#include "qdict.h"
+#include <QList>
 #include "MenuNode.h"
 
 class KMenuBar;
+class MenuSub;
 
 /**
  * This is the class for the root of a Menu (e.g. a MenuBar) that contains
@@ -44,14 +45,6 @@ public:
     /** Destructor */
     virtual ~MenuRoot();
 
-    /**
-     * Returns the positional index of a child node, identified by
-     * it's menu id.
-     * @param id the menu id of the child
-     * @return index [0..n] or -1 f not found
-     */
-    virtual int getChildIndex(int id);
-
     /**
      * Inserts a new branch node into the menu structure. The new node
      * normally is (derived from) MenuToplevel.
@@ -63,16 +56,14 @@ public:
      *                data from a child entry. (this is used for
      *                menus with data selection lists like "recent files)
      *                If not used, pass 0.
-     * @param key bitmask of the keyboard shortcut (see "qkeycode.h"),
-     *            0 if unused
+     * @param shortcut keyboard shortcut, 0 if unused
      * @param uid unique id string (might be 0)
-     * @param index the positional index within the parent menu, starting
-     *              from 0 or -1 for appending (optional, default=-1)
      * @return pointer to the new branch node
      */
-    virtual MenuNode *insertBranch(const QString &name,
-	const QString &command, int key, const QString &uid,
-	int index = -1);
+    virtual MenuSub *insertBranch(const QString &name,
+                                  const QString &command,
+                                  const QKeySequence &shortcut,
+                                  const QString &uid);
 
     /**
      * Inserts a new leaf node into the menu structure. The new node
@@ -80,15 +71,14 @@ public:
      * @param name non-localized name of the node
      * @param command the command to be sent when the node is
      *                selected (might be 0)
-     * @param key bitmask of the keyboard shortcut (see "qkeycode.h"),
-     *            0 if unused
+     * @param shortcut keyboard shortcut, 0 if unused
      * @param uid unique id string (might be 0)
-     * @param index the positional index within the parent menu, starting
-     *              from 0 or -1 for appending. Optional, default=-1
      * @return pointer to the new leaf node
      */
-    virtual MenuNode *insertLeaf(const QString &name, const QString &command,
-	int key, const QString &uid, int index = -1);
+    virtual MenuNode *insertLeaf(const QString &name,
+                                 const QString &command,
+                                 const QKeySequence &shortcut,
+                                 const QString &uid);
 
     /**
      * Removes a child node of the curren node. If the child
@@ -97,14 +87,6 @@ public:
      */
     virtual void removeChild(MenuNode *child);
 
-    /**
-     * Informs the node that the enabled state of a child node
-     * might have changed.
-     * @param id menu id of the child node
-     * @param enable true if the item has been enabled, false if disabled
-     */
-    virtual void actionChildEnableChanged(int id, bool enable);
-
     /**
      * Handles/interpretes special menu commands.
      * @param command name of a menu node or command
@@ -115,15 +97,15 @@ public:
     /**
      * Returns a pointer to the list of groups
      */
-    virtual QDict<MenuNode> *getGroupList();
+    virtual QHash<QString, MenuGroup *> &getGroupList();
 
-private:    // Private attributes
+private:
 
     /** reference to a KMenuBar */
     KMenuBar &m_menu_bar;
 
     /** list of menu groups */
-    QDict<MenuNode> m_group_list;
+    QHash<QString, MenuGroup *> m_group_list;
 
 };
 
