@@ -16,29 +16,25 @@
  ***************************************************************************/
 
 #include "config.h"
+
 #include <stdlib.h>
 #include <math.h>
 #include <stdio.h>
 
-#include <qpixmap.h>
-#include <qimage.h>
-#include <qdir.h>
-#include <qpainter.h>
-
-#include <kiconloader.h>
-#include <kstddirs.h>
+#include <QtGlobal>
+#include <QFont>
+#include <QPainter>
+#include <QPaintEvent>
 
 #include "ScaleWidget.h"
 
 #define FONTSIZE 6
 
 //***************************************************************************
-ScaleWidget::ScaleWidget(QWidget *parent, const char *name)
-    :QWidget(parent, name), m_low(0), m_high(100), m_logmode(false),
+ScaleWidget::ScaleWidget(QWidget *parent)
+    :QWidget(parent), m_low(0), m_high(100), m_logmode(false),
      m_unittext("%")
 {
-    KIconLoader icon_loader;
-    m_scalefont = icon_loader.loadIcon("minifont.xpm", KIcon::Small);
 }
 
 //***************************************************************************
@@ -47,14 +43,11 @@ ScaleWidget::ScaleWidget(QWidget *parent, int low, int high,
     :QWidget(parent), m_low(low), m_high(high), m_logmode(false),
      m_unittext(unit)
 {
-    KIconLoader icon_loader;
-    m_scalefont = icon_loader.loadIcon("minifont.xpm", KIcon::Small);
 }
 
 //***************************************************************************
 ScaleWidget::~ScaleWidget()
 {
-    m_scalefont.resize(0,0);
 }
 
 //***************************************************************************
@@ -82,36 +75,29 @@ void ScaleWidget::setMinMax(int min, int max)
 }
 
 //***************************************************************************
-void ScaleWidget::paintText(QPainter &p, int x, int y, int ofs,
+void ScaleWidget::paintText(QPainter &p, int x, int y,
                             bool reverse, const QString &text)
 {
-    int len = text.length();
-    int pos; // position within the font bitmap
+    QFont font;
+    font.setStyleHint(QFont::SansSerif);
+    font.setFixedPitch(true);
+    font.setPixelSize(FONTSIZE);
+    font.setWeight(0);
+    font.setStyle(QFont::StyleNormal);
+    p.setFont(font);
+    QFontMetrics fm(font);
 
-    if (reverse) x -= ofs;
-    for (int i = 0; i < len; i++) {
-	pos = (reverse) ? (len-1-i) : i;
-	int c = text.at(pos).latin1();
-
-	pos = 40; // default = space
-	switch (c) {
-	    case '\'':  pos = 36; break;
-	    case '"':   pos = 37; break;
-	    case 176:   pos = 38; break;
-	    case '.':   pos = 39; break;
-	    case ' ':   pos = 40; break;
-	    case '%':   pos = 41; break;
-	    case '-':   pos = 42; break;
-	    default:
-		if ((c > 64) && (c < 91))  pos = c-65;    // letter
-		if ((c > 96) && (c < 123)) pos = c-97;    // letter
-		if ((c > 47) && (c < 58))  pos = c-48+26; // number
-	}
-
-	p.drawPixmap(x, y, m_scalefont, pos*FONTSIZE, 0, FONTSIZE, FONTSIZE);
-
-	x += (reverse) ? (-ofs) : (+ofs);
+    if (reverse) {
+	x += FONTSIZE + 2;
+	y -= FONTSIZE + 2;
     }
+
+    QRect rect = fm.boundingRect(text);
+    const int th = rect.height();
+    const int tw = rect.width();
+    p.drawText(x, y, tw, th,
+	((reverse) ? Qt::AlignLeft : Qt::AlignRight) | Qt::AlignBottom,
+	text);
 }
 
 //***************************************************************************
@@ -123,11 +109,11 @@ void ScaleWidget::drawLog(QPainter &p, int w, int h, bool inverse)
 
     int dir = (inverse) ? -1 : +1;
 
-    p.setPen (colorGroup().dark());
+    p.setPen(palette().dark().color());
     p.drawLine (0, dir*(h-1), dir*w, dir*(h-1));
     p.drawLine (dir*(w-1), 0, dir*(w-1), dir*(h-1));
 
-    p.setPen (colorGroup().text());
+    p.setPen(palette().text().color());
 
     int a, x;
     const int h2 = h;
@@ -137,7 +123,7 @@ void ScaleWidget::drawLog(QPainter &p, int w, int h, bool inverse)
 
     int dec_lo = (m_low) ? (int)floor(log(m_low)/log(base)) : 0;
     int dec_hi = (int)ceil(log(m_high)/log(base));
-    int decades = abs(dec_hi - dec_lo) + 1;
+    int decades = qAbs(dec_hi - dec_lo) + 1;
 
     // check if we have enough space for the small lines within a decade
     int w1 = (int)(w / decades); // pixels per decade
@@ -165,7 +151,7 @@ void ScaleWidget::drawLog(QPainter &p, int w, int h, bool inverse)
 	int value = (int)pow(base, dec_lo+a);
 	buf = buf.arg(value).arg(m_unittext);
 	x = ((w-1) * a)/decades;
-	paintText(p, dir*(x+2), dir*(h-FONTSIZE-2), FONTSIZE, inverse, buf);
+	paintText(p, dir*(x+4), dir*(h-FONTSIZE-4), inverse, buf);
     }
 }
 
@@ -174,11 +160,11 @@ void ScaleWidget::drawLinear(QPainter &p, int w, int h, bool inverse)
 {
     int dir = (inverse) ? -1 : +1;
 
-    p.setPen(colorGroup().dark());
+    p.setPen(palette().dark().color());
     p.drawLine(0, dir*(h-1), dir*w, dir*(h-1));
     p.drawLine(dir*(w-1), 0, dir*(w-1), dir*(h - 1));
 
-    p.setPen (colorGroup().text());
+    p.setPen(palette().text().color());
 
     int a, x;
     double ofs;
@@ -203,7 +189,7 @@ void ScaleWidget::drawLinear(QPainter &p, int w, int h, bool inverse)
 	int value = m_low + (((m_high - m_low)* (inverse ? (4-a) : a))/4);
 	buf = buf.arg(value).arg(m_unittext);
 	x = ((w-1) * a)/4;
-	paintText(p, dir*(x+2), dir*(h-FONTSIZE-2), FONTSIZE, inverse, buf);
+	paintText(p, dir*(x+4), dir*(h-FONTSIZE-4), inverse, buf);
     }
 
 }
@@ -218,7 +204,7 @@ void ScaleWidget::paintEvent(QPaintEvent *)
 
     p.begin(this);
     p.save();
-    p.setPen(colorGroup().light());
+    p.setPen(palette().light().color());
 
     p.drawLine(0, 0, w, 0);
     if (h > w) {
@@ -237,13 +223,13 @@ void ScaleWidget::paintEvent(QPaintEvent *)
 }
 
 //***************************************************************************
-const QSize ScaleWidget::sizeHint()
+QSize ScaleWidget::sizeHint() const
 {
     return QSize(4*FONTSIZE, 4*FONTSIZE);
 }
 
 //***************************************************************************
-const QSize ScaleWidget::minimumSize()
+QSize ScaleWidget::minimumSize() const
 {
     return QSize(5*2*FONTSIZE, 5*2*FONTSIZE);
 }
