@@ -20,10 +20,12 @@
 
 #include "config.h"
 
-#include <qmap.h>
-#include <qstring.h>
+#include <kdemacros.h>
 
-class GenreType
+#include <QMap>
+#include <QString>
+
+class KDE_EXPORT GenreType
 {
 private:
 
