@@ -17,8 +17,11 @@
 
 #include "config.h"
 
-#include <qbuffer.h>
+#include <QBuffer>
+#include <QMimeData>
+#include <QMimeSource>
 
+#include "libkwave/CodecManager.h"
 #include "libkwave/Decoder.h"
 #include "libkwave/Encoder.h"
 #include "libkwave/KwaveDrag.h"
@@ -26,17 +29,16 @@
 #include "libkwave/SampleReader.h"
 #include "libkwave/SampleWriter.h"
 #include "libkwave/Signal.h"
+#include "libkwave/SignalManager.h"
 #include "libkwave/MultiTrackReader.h"
 #include "libkwave/MultiTrackWriter.h"
-#include "kwave/CodecManager.h"
-#include "kwave/SignalManager.h"
 
 // RFC 2361:
 #define WAVE_FORMAT_PCM "audio/vnd.wave" // ; codec=001"
 
 //***************************************************************************
-KwaveDrag::KwaveDrag(QWidget *dragSource, const char *name)
-    :QDragObject(dragSource, name), m_data()
+KwaveDrag::KwaveDrag(QWidget *dragSource)
+    :QDrag(dragSource), m_data()
 {
 }
 
@@ -59,17 +61,16 @@ const char *KwaveDrag::format(int i) const
 QByteArray KwaveDrag::encodedData(const char *format) const
 {
     qDebug("KwaveDrag::encodedData(%s)", format);
-    if (QCString(WAVE_FORMAT_PCM) == QCString(format)) return m_data;
+    if (QString(WAVE_FORMAT_PCM) == QString(format)) return m_data;
     return QByteArray();
 }
 
 //***************************************************************************
-bool KwaveDrag::canDecode(const QMimeSource* e)
+bool KwaveDrag::canDecode(const QMimeData *data)
 {
-    if (!e) return false;
-    for (int i=0; e->format(i); ++i) {
-	if (CodecManager::canDecode(e->format(i))) return true;
-    }
+    if (!data) return false;
+    foreach (QString format, data->formats())
+	if (CodecManager::canDecode(format)) return true;
     return false;
 }
 
@@ -81,19 +82,32 @@ bool KwaveDrag::encode(QWidget *widget, MultiTrackReader &src, FileInfo &info)
     Q_ASSERT(src[0]);
     if (!src[0]) return false;
 
+    // create a mime data container
+    QMimeData *mime_data = new QMimeData;
+    Q_ASSERT(mime_data);
+    if (!mime_data) return false;
+
     // use our default encoder
     Encoder *encoder = CodecManager::encoder(WAVE_FORMAT_PCM);
     Q_ASSERT(encoder);
-    if (!encoder) return false;
+    if (!encoder) {
+	delete mime_data;
+	return false;
+    }
 
     // create a buffer for the wav data
     m_data.resize(0);
-    QBuffer dst(m_data);
+    QBuffer dst(&m_data);
 
     // encode into the buffer
     encoder->encode(widget, src, dst, info);
 
     delete encoder;
+
+    // set the mime data into this drag&drop container
+    mime_data->setData("audio/vnd.wave", m_data);
+    setMimeData(mime_data);
+
     return true;
 }
 
@@ -110,22 +124,37 @@ unsigned int KwaveDrag::decode(QWidget *widget, const QMimeSource *e,
     int i;
     const char *format;
     unsigned int decoded_length = 0;
+    unsigned int decoded_tracks = 0;
 
     for (i=0; (format = e->format(i)); ++i) {
 	if (CodecManager::canDecode(format)) {
-	    QBuffer src(e->encodedData(format));
+	    QByteArray raw_data = e->encodedData(format);
+	    QBuffer src(&raw_data);
 
 	    // open the mime source and get header information
 	    bool ok = decoder->open(widget, src);
 	    if (!ok) continue;
 	    decoded_length = decoder->info().length();
+	    decoded_tracks = decoder->info().tracks();
 	    Q_ASSERT(decoded_length);
-	    if (!decoded_length) continue;
+	    Q_ASSERT(decoded_tracks);
+	    if (!decoded_length || !decoded_tracks) continue;
+
+	    if (!sig.tracks()) {
+		// drop into an empty window -> create tracks
+		qDebug("KwaveDrag::decode(...) -> new signal");
+		sig.newSignal(0,
+		    decoder->info().rate(),
+		    decoder->info().bits(),
+		    decoded_tracks);
+		ok = (sig.tracks() == decoded_tracks);
+		if (!ok) continue;
+	    }
 
 	    // prepare the signal
 	    unsigned int left  = pos;
 	    unsigned int right = left + decoded_length - 1;
-	    MultiTrackWriter dst(sig, sig.allTracks(), Insert,
+	    MultiTrackWriter dst(sig, sig.selectedTracks(), Insert,
 	                         left, right);
 
 	    ok = decoder->decode(widget, dst);
