@@ -17,46 +17,26 @@
 #ifndef _LABEL_LIST_H_
 #define _LABEL_LIST_H_
 
-#include <config.h>
-#include <qptrlist.h>
+#include "config.h"
+
+#include <QList>
+#include <QListIterator>
+
+#include <kdemacros.h>
 
 #include "Label.h"
 
-class LabelList: public QPtrList<Label>
+/** not more than a typedef since Qt4 */
+class KDE_EXPORT LabelList: public QList<Label *>
 {
 public:
-    /** Constructor */
-    LabelList();
 
-    /** Destructor */
-    virtual ~LabelList();
+    /** sorts the list by ascending position */
+    void sort();
 
-    /** Copy everything from another LabelList */
-    void copy(const LabelList &source);
-
-    /** Return true if the list is equal to another list */
-    bool equals(const LabelList &other) const;
-
-    /** Assignment operator */
-    inline LabelList & operator = (const LabelList &source) {
-	copy(source);
-	return *this;
-    };
-
-    /** Compare operator */
-    inline bool operator == (const LabelList &other) const {
-	return equals(other);
-    }
-
-protected:
-    /**
-     * compare two labels, for sorting
-     * @see QPtrList::compareItems
-     */
-    virtual int compareItems(QPtrCollection::Item a, QPtrCollection::Item b);
 };
 
 /** Iterator for the list of labels */
-typedef QPtrListIterator<Label> LabelListIterator;
+typedef QListIterator<Label *> LabelListIterator;
 
 #endif /* _LABEL_LIST_H_ */
