@@ -16,35 +16,38 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "config.h"
+
 #include "libkwave/MultiTrackReader.h"
 #include "libkwave/SampleReader.h"
-#include "kwave/SignalManager.h"
+#include "libkwave/SignalManager.h"
 
 //***************************************************************************
 MultiTrackReader::MultiTrackReader()
     :Kwave::MultiTrackSource<SampleReader, false>(0,0),
-     m_cancelled(false)
+     m_canceled(false)
 {
 }
 
 //***************************************************************************
 MultiTrackReader::MultiTrackReader(SignalManager &signal_manager,
-    const QMemArray<unsigned int> &track_list,
-    unsigned int first, unsigned int last)
-    :Kwave::MultiTrackSource<SampleReader, false>(
-	track_list.count(), 0, "MultiTrackReader"),
-     m_cancelled(false)
+                                   const QList<unsigned int> &track_list,
+                                   unsigned int first, unsigned int last)
+    :Kwave::MultiTrackSource<SampleReader, false>(0),
+     m_canceled(false)
 {
-    unsigned int count = track_list.count();
-    for (unsigned int i=0; i < count; i++) {
-	unsigned int track = track_list[i];
-	SampleReader *s =
-	    signal_manager.openSampleReader(track, first, last);
+    const unsigned int count = track_list.count();
+    unsigned int index = 0;
+
+    foreach(unsigned int track, track_list) {
+	SampleReader *s = signal_manager.openSampleReader(track, first, last);
 	Q_ASSERT(s);
-	insert(i, s);
+	insert(index++, s);
+	Q_ASSERT(index == tracks());
     }
 
     Q_ASSERT(count == tracks());
+    Q_UNUSED(count);
 }
 
 //***************************************************************************
@@ -56,9 +59,9 @@ MultiTrackReader::~MultiTrackReader()
 //***************************************************************************
 bool MultiTrackReader::eof() const
 {
-    unsigned int c = tracks();
+    const unsigned int c = tracks();
     for (unsigned int r = 0; r < c; r++) {
-	SampleReader *reader = this->at(r);
+	SampleReader *reader = at(r);
 	Q_ASSERT(reader);
 	if (!reader) continue;
 	if (reader->eof()) return true;
@@ -73,7 +76,7 @@ void MultiTrackReader::proceeded()
     unsigned int track;
     const unsigned int n_tracks = tracks();
     for (track=0; track < n_tracks; ++track) {
-	SampleReader *r = (*this)[track];
+	SampleReader *r = at(track);
 	if (r) pos += (r->pos() - r->first());
     }
     emit progress(pos);
@@ -86,16 +89,26 @@ void MultiTrackReader::reset()
     unsigned int track;
     const unsigned int n_tracks = tracks();
     for (track=0; track < n_tracks; ++track) {
-	SampleReader *r = (*this)[track];
+	SampleReader *r = at(track);
 	if (r) r->reset();
     }
     emit progress(pos);
 }
 
+//***************************************************************************
+bool MultiTrackReader::insert(unsigned int track, SampleReader *reader)
+{
+    if (reader) {
+        connect(reader, SIGNAL(proceeded()), this, SLOT(proceeded()));
+    }
+    return Kwave::MultiTrackSource<SampleReader, false>::insert(
+        track, reader);
+}
+
 //***************************************************************************
 void MultiTrackReader::cancel()
 {
-    m_cancelled = true;
+    m_canceled = true;
 }
 
 //***************************************************************************
