@@ -20,8 +20,12 @@
 #define _MULTI_TRACK_WRITER_H_
 
 #include "config.h"
-#include <qobject.h>
-#include <qptrvector.h>
+
+#include <QObject>
+#include <QList>
+
+#include <kdemacros.h>
+
 #include "libkwave/SampleWriter.h"
 #include "libkwave/KwaveMultiTrackSink.h"
 
@@ -32,7 +36,7 @@ class SignalManager;
  * A MultiTrackWriter encapsulates a set of <c>SampleWriter</c>s for
  * easier use of multi-track signals.
  */
-class MultiTrackWriter:
+class KDE_EXPORT MultiTrackWriter:
     public Kwave::MultiTrackSink<SampleWriter>
 {
     Q_OBJECT
@@ -51,7 +55,7 @@ public:
      * @param right index of the last sample
      */
     MultiTrackWriter(SignalManager &signal_manager,
-                     const QMemArray<unsigned int> &track_list,
+                     const QList<unsigned int> &track_list,
                      InsertMode mode,
                      unsigned int left, unsigned int right);
 
@@ -79,19 +83,19 @@ public:
     MultiTrackWriter &operator << (MultiTrackReader &source);
 
     /** Returns the last sample index of all streams */
-    unsigned int last();
+    unsigned int last() const;
 
     /** Flushes all streams */
     void flush();
 
-    /** @see QPtrVector::clear() */
+    /** @see QList::clear() */
     virtual void clear();
 
-    /** @see QPtrVector::insert() */
+    /** @see QList::insert() */
     virtual bool insert(unsigned int track, SampleWriter *writer);
 
-    /** returns true if the transfer has been cancelled */
-    inline bool isCancelled() { return m_cancelled; };
+    /** returns true if the transfer has been canceled */
+    inline bool isCanceled() const { return m_canceled; };
 
 signals:
 
@@ -120,9 +124,9 @@ protected:
 
     /**
      * Initialized as false, will be true if the transfer has
-     * been cancelled
+     * been canceled
      */
-    bool m_cancelled;
+    bool m_canceled;
 
 };
 
