@@ -17,19 +17,18 @@
 
 #include "config.h"
 
-#include <qstringlist.h>
+#include <QStringList>
 #include <klocale.h>
 
-#include "libkwave/CurveStreamAdapter.h"
 #include "libkwave/KwaveConnect.h"
-#include "libkwave/KwaveMul.h"
 #include "libkwave/MultiTrackReader.h"
 #include "libkwave/MultiTrackWriter.h"
 #include "libkwave/KwaveMultiTrackSource.h"
 #include "libkwave/Parser.h"
-
-#include "kwave/PluginManager.h"
-#include "kwave/UndoTransactionGuard.h"
+#include "libkwave/PluginManager.h"
+#include "libkwave/modules/CurveStreamAdapter.h"
+#include "libkwave/modules/KwaveMul.h"
+#include "libkwave/undo/UndoTransactionGuard.h"
 
 #include "AmplifyFreePlugin.h"
 #include "AmplifyFreeDialog.h"
@@ -57,7 +56,7 @@ int AmplifyFreePlugin::interpreteParameters(QStringList &params)
     // convert string list into command again...
     QString cmd;
     cmd = "curve(";
-    for (unsigned int i=0; i < params.count(); ++i) {
+    for (int i=0; i < params.count(); ++i) {
 	cmd += params[i];
 	if (i+1 < params.count()) cmd += ",";
     }
@@ -117,7 +116,7 @@ void AmplifyFreePlugin::run(QStringList params)
     Kwave::CurveStreamAdapter curve(m_curve, input_length);
     MultiTrackWriter sink(signalManager(), selectedTracks(), Overwrite,
 	first, last);
-    Kwave::MultiTrackSource<Kwave::Mul, true> mul(tracks, this, "AmplifyFree");
+    Kwave::MultiTrackSource<Kwave::Mul, true> mul(tracks, this);
 
     // connect them
     bool ok = true;
