@@ -17,10 +17,11 @@
 
 #include "config.h"
 #include "errno.h"
-#include <kmessagebox.h>
 
-#include "kwave/PluginManager.h"
-#include "kwave/SignalManager.h"
+#include "libkwave/MessageBox.h"
+#include "libkwave/PluginManager.h"
+#include "libkwave/SignalManager.h"
+
 #include "FileInfoDialog.h"
 #include "FileInfoPlugin.h"
 
@@ -72,14 +73,15 @@ void FileInfoPlugin::apply(FileInfo &new_info)
     if (fileInfo().rate() != new_info.rate()) {
 	// sample rate changed -> only change rate or resample ?
 	double new_rate = new_info.rate();
-	int res = KMessageBox::questionYesNoCancel(parentWidget(),
+	int res = Kwave::MessageBox::questionYesNoCancel(parentWidget(),
 	    i18n("You have changed the sample rate. Do you want to convert "
 		 "the whole file to the new sample rate or do "
 		 "you only want to set the rate information in order "
 		 "to repair a damaged file? Note: changing only the sample "
 		 "rate can cause \"mickey mouse\" effects!"),
 	    0,
-	    i18n("&Convert"), i18n("&Set Rate"));
+	    i18n("&Convert"),
+	    i18n("&Set Rate"));
 	if (res == KMessageBox::Yes) {
 	    // resample
 	    emitCommand(QString("convert_rate(%1)").arg(new_rate));
@@ -87,7 +89,7 @@ void FileInfoPlugin::apply(FileInfo &new_info)
 	    // change the rate only
 	    // fileInfo().setRate(new_rate);
 	} else {
-	    // cancelled -> use old setting
+	    // canceled -> use old setting
 	    new_info.setRate(fileInfo().rate());
 	}
     }
