@@ -16,9 +16,12 @@
  ***************************************************************************/
 
 #include "config.h"
+
 #include <math.h>
 #include <errno.h>
-#include <qstringlist.h>
+
+#include <QStringList>
+
 #include <klocale.h>
 
 #include "libkwave/KwaveMultiTrackSource.h"
