@@ -15,6 +15,8 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "config.h"
+
 #include <klocale.h> // for the i18n macro
 
 #include "libkwave/MultiTrackWriter.h"
@@ -22,9 +24,8 @@
 #include "libkwave/KwaveSampleSource.h"
 #include "libkwave/KwaveSampleSink.h"
 #include "libkwave/KwaveMultiTrackSource.h"
-
-#include "kwave/PluginManager.h"
-#include "kwave/UndoTransactionGuard.h"
+#include "libkwave/PluginManager.h"
+#include "libkwave/undo/UndoTransactionGuard.h"
 
 #include "NoisePlugin.h"
 #include "NoiseGenerator.h"
@@ -62,7 +63,7 @@ void NoisePlugin::run(QStringList)
     }
 
     // transport the samples
-    qDebug("NoisePlugin: filter started...");
+    qDebug("NoisePlugin: filter started [%u ... %u] ...", first, last);
     while (!m_stop && !(sink.done())) {
 	source.goOn();
     }
