@@ -19,13 +19,14 @@
 #include <errno.h>
 #include <math.h>
 
-#include <qstring.h>
+#include <QString>
 #include <klocale.h>
 
-#include "kwave/PluginManager.h"
-#include "kwave/SignalManager.h"
-#include "kwave/UndoTransactionGuard.h"
 #include "libkwave/KwavePlugin.h"
+#include "libkwave/PluginManager.h"
+#include "libkwave/SignalManager.h"
+#include "libkwave/undo/UndoTransactionGuard.h"
+
 #include "SelectRangePlugin.h"
 #include "SelectRangeDialog.h"
 
