@@ -16,13 +16,14 @@
  ***************************************************************************/
 
 #include "config.h"
+
 #include <math.h>
 #include <limits.h>
 
-#include <qbitmap.h>
-#include <qimage.h>
-#include <qlayout.h>
-#include <qtimer.h>
+#include <QBitmap>
+#include <QImage>
+#include <QLayout>
+#include <QTimer>
 
 #include <kmenubar.h>
 #include <kfiledialog.h>
@@ -30,9 +31,11 @@
 
 #include "libkwave/KwavePlugin.h"
 #include "libkwave/WindowFunction.h"
+
+#include "libgui/ImageView.h"
+#include "libgui/KwaveFileDialog.h"
 #include "libgui/ScaleWidget.h"
 
-#include "ImageView.h"
 #include "SonagramWindow.h"
 
 /**
@@ -40,14 +43,6 @@
  */
 #define REFRESH_DELAY 100
 
-#ifndef min
-#define min(x,y) (( (x) < (y) ) ? (x) : (y) )
-#endif
-
-#ifndef max
-#define max(x,y) (( (x) > (y) ) ? (x) : (y) )
-#endif
-
 /**
  * Color values below this limit are cut off when adjusting the
  * sonagram image's brightness
@@ -86,24 +81,19 @@ static const char *background[] = {
 
 //****************************************************************************
 SonagramWindow::SonagramWindow(const QString &name)
-    :KMainWindow()
+    :KMainWindow(), m_image(), m_color_mode(0), m_view(0), m_overview(0),
+     m_points(0), m_rate(0), m_xscale(0), m_yscale(0), m_refresh_timer()
 {
-    m_color_mode = 0;
-    m_image = 0;
-    m_overview = 0;
-    m_points = 0;
-    m_rate = 0;
-    m_view = 0;
-    m_xscale = 0;
-    m_yscale = 0;
-    for (int i=0; i<256; m_histogram[i++] = 0) {}
+    KIconLoader icon_loader;
+
+    for (int i = 0; i < 256; m_histogram[i++] = 0) {}
 
     QWidget *mainwidget = new QWidget(this);
     Q_ASSERT(mainwidget);
     if (!mainwidget) return;
     setCentralWidget(mainwidget);
 
-    QGridLayout *top_layout = new QGridLayout(mainwidget, 3, 2);
+    QGridLayout *top_layout = new QGridLayout(mainwidget/*, 3, 2*/);
     Q_ASSERT(top_layout);
     if (!top_layout) return;
 
@@ -111,22 +101,30 @@ SonagramWindow::SonagramWindow(const QString &name)
     Q_ASSERT(bar);
     if (!bar) return ;
 
-//    QPopupMenu *spectral = new QPopupMenu();
+//    QMenu *spectral = new QMenu();
 //    Q_ASSERT(spectral);
 //    if (!spectral) return ;
 
-    QPopupMenu *file = new QPopupMenu();
+    QMenu *file = bar->addMenu(i18n("&Sonagram"));
     Q_ASSERT(file);
     if (!file) return ;
 
-    bar->insertItem(i18n("&Sonagram"), file);
-//    bar->insertItem(i18n("&Spectral Data"), spectral);
-//
-//    file->insertItem(i18n("&Import from Bitmap ..."), this, SLOT(load()));
-    file->insertItem(i18n("&Export to Bitmap ..."), this, SLOT(save()));
-    file->insertItem(i18n("E&xit"), this, SLOT(close()));
-//
-//    spectral->insertItem (i18n("&reTransform to signal"), this, SLOT(toSignal()));
+//    bar->addAction(i18n("&Spectral Data"), spectral);
+//    file->addAction(i18n("&Import from Bitmap ..."), this, SLOT(load()));
+
+    file->addAction(
+	icon_loader.loadIcon("document-export", KIconLoader::Small),
+	i18n("&Export to Bitmap ..."),
+	this, SLOT(save())
+    );
+    file->addAction(
+	icon_loader.loadIcon("dialog-close", KIconLoader::Small),
+	i18n("&Close"),
+	this, SLOT(close()),
+	QKeySequence::Close
+    );
+
+//    spectral->addAction (i18n("&reTransform to signal"), this, SLOT(toSignal()));
 
     KStatusBar *status = statusBar();
     Q_ASSERT(status);
@@ -140,7 +138,9 @@ SonagramWindow::SonagramWindow(const QString &name)
     Q_ASSERT(m_view);
     if (!m_view) return;
     top_layout->addWidget(m_view, 0, 1);
-    m_view->setBackgroundPixmap(QPixmap(background));
+    QPalette palette;
+    palette.setBrush(m_view->backgroundRole(), QBrush(background));
+    m_view->setPalette(palette);
 
     m_xscale = new ScaleWidget(mainwidget, 0, 100, "ms");
     Q_ASSERT(m_xscale);
@@ -171,8 +171,8 @@ SonagramWindow::SonagramWindow(const QString &name)
     top_layout->setRowStretch(0, 100);
     top_layout->setRowStretch(1, 0);
     top_layout->setRowStretch(2, 0);
-    top_layout->setColStretch(0, 0);
-    top_layout->setColStretch(1, 100);
+    top_layout->setColumnStretch(0, 0);
+    top_layout->setColumnStretch(1, 100);
     top_layout->activate();
 
     status->changeItem(i18n("Time: 0 ms"), 1);
@@ -198,11 +198,16 @@ void SonagramWindow::close()
 //****************************************************************************
 void SonagramWindow::save()
 {
-    Q_ASSERT(m_image);
-    if (!m_image) return;
+    if (m_image.isNull()) return;
 
-    QString filename = KFileDialog::getSaveFileName("", "*.bmp", this);
-    if (!filename.isEmpty()) m_image->save(filename, "BMP");
+    KwaveFileDialog dlg(":<kwave_sonagram>", QString(),
+        this, true, QString(), "*.bmp");
+    dlg.setOperationMode(KFileDialog::Saving);
+    dlg.setCaption(i18n("Save Sonagram"));
+    if (dlg.exec() != QDialog::Accepted) return;
+    QString filename = dlg.selectedFile();
+
+    if (!filename.isEmpty()) m_image.save(filename, "BMP");
 }
 
 //****************************************************************************
@@ -246,7 +251,7 @@ void SonagramWindow::load()
 }
 
 //****************************************************************************
-void SonagramWindow::setImage(QImage *image)
+void SonagramWindow::setImage(QImage image)
 {
     Q_ASSERT(m_view);
     if (!m_view) return;
@@ -254,31 +259,24 @@ void SonagramWindow::setImage(QImage *image)
     m_image = image;
 
     // re-initialize histogram over all pixels
-    for (int i=0; i<256; i++) m_histogram[i] = 0;
-    if (m_image) {
-	for (int x=0; x < m_image->width(); x++) {
-	    for (int y=0; y < m_image->height(); y++) {
-		unsigned char p = m_image->pixelIndex(x, y);
+    for (int i=0; i < 256; i++)
+	m_histogram[i] = 0;
+    if (!m_image.isNull()) {
+	for (int x = 0; x < m_image.width(); x++) {
+	    for (int y = 0; y < m_image.height(); y++) {
+		unsigned char p = m_image.pixelIndex(x, y);
 		m_histogram[p]++;
 	    }
 	}
     }
 
-    m_view->setImage(m_image);
     refresh_view();
 }
 
 //****************************************************************************
-void SonagramWindow::setOverView(QBitmap *overview)
+void SonagramWindow::setOverView(QBitmap overview)
 {
-    QImage *image = 0;
-    if (!m_overview) return;
-    if (overview) {
-	image = new QImage(overview->convertToImage());
-	Q_ASSERT(image);
-    }
-    m_overview->setImage(image);
-    if (image) delete image;
+    m_overview->setImage(overview.toImage());
 }
 
 //****************************************************************************
@@ -286,55 +284,54 @@ void SonagramWindow::insertStripe(const unsigned int stripe_nr,
 	const QByteArray &stripe)
 {
     Q_ASSERT(m_view);
-    Q_ASSERT(m_image);
+    Q_ASSERT(!m_image.isNull());
     if (!m_view) return;
-    if (!m_image) return;
+    if (m_image.isNull()) return;
 
-    unsigned int image_width  = m_image->width();
-    unsigned int image_height = m_image->height();
+    unsigned int image_width  = m_image.width();
+    unsigned int image_height = m_image.height();
 
     // stripe is out of range ?
-    Q_ASSERT(stripe_nr < image_width);
-    if ((stripe_nr) >= image_width) return;
+    if (stripe_nr >= image_width) return;
 
     unsigned int y;
     unsigned int size = stripe.size();
-    for (y=0; y < size; y++) {
-    	unsigned char p;
+    for (y = 0; y < size; y++) {
+	unsigned char p;
 
-    	// remove the current pixel from the histogram
-    	p = m_image->pixelIndex(stripe_nr, y);
+	// remove the current pixel from the histogram
+	p = m_image.pixelIndex(stripe_nr, y);
 	m_histogram[p]--;
 
 	// set the new pixel value
-	p = (unsigned char)stripe[size-y-1];
-	m_image->setPixel(stripe_nr, y, p);
+	p = (int)stripe[(size - 1) - y];
+	m_image.setPixel(stripe_nr, y, p);
 
-    	// insert the new pixel into the histogram
+	// insert the new pixel into the histogram
 	m_histogram[p]++;
     }
     while (y < image_height) { // fill the rest with blank
-	m_image->setPixel(stripe_nr, y++, 0xFE);
+	m_image.setPixel(stripe_nr, y++, 0xFE);
 	m_histogram[0xFE]++;
     }
 
     if (!m_refresh_timer.isActive()) {
-	m_refresh_timer.start(REFRESH_DELAY, true);
+	m_refresh_timer.setSingleShot(true);
+	m_refresh_timer.start(REFRESH_DELAY);
     }
 }
 
 //****************************************************************************
 void SonagramWindow::adjustBrightness()
 {
-    Q_ASSERT(m_image);
-    if (!m_image) return;
+    if (m_image.isNull()) return;
 
     // get the sum of pixels != 0
     unsigned long int sum = 0;
-    for (int i=0; i<254; i++) {
+    for (int i=0; i < 254; i++) {
 	sum += m_histogram[i];
     }
-    // cut off all parts below the cutoff ration (e.g. 0.1%)
+    // cut off all parts below the cutoff ratio (e.g. 0.1%)
     unsigned int cutoff = (int)(sum * COLOR_CUTOFF_RATIO);
 
     // get the first used color
@@ -362,12 +359,15 @@ void SonagramWindow::adjustBrightness()
 	    c.setRgb(v, v, v);
 	}
 
-	m_image->setColor(i, c.rgb() | 0xFF000000);
+	c.setAlpha(255);
+	m_image.setColor(i, c.rgb());
+// 	qDebug("color[%3d] = 0x%08X",i, c.rgb());
     }
 
     // use color 0xFF for transparency !
-    m_image->setColor(0xFF, 0x00000000);
-
+    c.setRgb(0,0,0);
+    c.setAlpha(0);
+    m_image.setColor(255, c.rgb());
 }
 
 //****************************************************************************
@@ -375,8 +375,8 @@ void SonagramWindow::refresh_view()
 {
     Q_ASSERT(m_view);
     if (!m_view) return;
-    if (m_image) adjustBrightness();
-    m_view->repaint(false);
+    adjustBrightness();
+    m_view->setImage(m_image);
 }
 
 //****************************************************************************
@@ -471,7 +471,7 @@ void SonagramWindow::updateScaleWidgets()
     double ms;
     double f;
 
-    translatePixels2TF(QPoint(m_image->width()-1, 0), &ms, &f);
+    translatePixels2TF(QPoint(m_image.width()-1, 0), &ms, &f);
 
     m_xscale->setMinMax(0, (int)rint(ms));
     m_yscale->setMinMax(0, (int)rint(f));
@@ -490,7 +490,7 @@ void SonagramWindow::setColorMode(int mode)
 
     if (mode != m_color_mode) {
 	m_color_mode = mode;
-	if (m_image) setImage(m_image);
+	setImage(m_image);
     }
 }
 
@@ -509,37 +509,34 @@ void SonagramWindow::cursorPosChanged(const QPoint pos)
 {
     KStatusBar *status = statusBar();
     Q_ASSERT(status);
-    Q_ASSERT(m_image);
     Q_ASSERT(m_points);
     Q_ASSERT(m_rate != 0);
-    if (!status) return ;
-    if (!m_image) return ;
+    if (!status) return;
+    if (m_image.isNull()) return;
     if (!m_points) return;
     if (m_rate == 0) return;
 
-    char buf[64];
     double ms;
     double f;
     double a;
     translatePixels2TF(pos, &ms, &f);
 
     // item 1: time in milliseconds
-    status->changeItem(i18n("Time: %1").arg(
-	KwavePlugin::ms2string(ms)), 1);
+    status->changeItem(i18n("Time: %1", KwavePlugin::ms2string(ms)), 1);
 
     // item 2: frequency in Hz
-    snprintf(buf, sizeof(buf), i18n("Frequency: %d Hz"), (int)f);
-    status->changeItem(buf, 2);
+    QString text = i18n("Frequency: %1 Hz", (int)f);
+    status->changeItem(text, 2);
 
     // item 3: amplitude in %
-    if (m_image->valid(pos.x(), pos.y())) {
-	a = (254.0-m_image->pixelIndex(pos.x(), pos.y())) *
+    if (m_image.valid(pos.x(), pos.y())) {
+	a = (254.0 - m_image.pixelIndex(pos.x(), pos.y())) *
 	    (100.0 / 254.0);
     } else {
 	a = 0.0;
     }
-    snprintf(buf, sizeof(buf), i18n("Amplitude: %d %%"), (int)a);
-    status->changeItem(buf, 3);
+    text = i18n("Amplitude: %1 %", (int)a);
+    status->changeItem(text, 3);
 }
 
 //****************************************************************************
