@@ -18,18 +18,18 @@
 #include "config.h"
 #include <errno.h>
 
-#include <qstringlist.h>
+#include <QStringList>
+
 #include <klocale.h>
 
 #include "libkwave/KwaveConnect.h"
-#include "libkwave/KwaveMul.h"
 #include "libkwave/MultiTrackReader.h"
 #include "libkwave/MultiTrackWriter.h"
 #include "libkwave/KwaveMultiTrackSource.h"
 #include "libkwave/Parser.h"
-
-#include "kwave/PluginManager.h"
-#include "kwave/UndoTransactionGuard.h"
+#include "libkwave/PluginManager.h"
+#include "libkwave/modules/KwaveMul.h"
+#include "libkwave/undo/UndoTransactionGuard.h"
 
 #include "VolumePlugin.h"
 #include "VolumeDialog.h"
@@ -117,23 +117,18 @@ void VolumePlugin::run(QStringList params)
     MultiTrackReader source(signalManager(), selectedTracks(), first, last);
     MultiTrackWriter sink(signalManager(), selectedTracks(), Overwrite,
 	first, last);
-    Kwave::MultiTrackSource<Kwave::Mul, true> mul(tracks, this, "Volume");
+    Kwave::MultiTrackSource<Kwave::Mul, true> mul(tracks);
 
     // connect them
-    bool ok = true;
-    if (ok) ok = Kwave::connect(
+    Kwave::connect(
 	source, SIGNAL(output(Kwave::SampleArray &)),
 	mul,    SLOT(input_a(Kwave::SampleArray &)));
+
     mul.setAttribute(SLOT(set_b(const QVariant &)),
                      QVariant(m_factor));
-    if (ok) ok = Kwave::connect(
+    Kwave::connect(
 	mul,    SIGNAL(output(Kwave::SampleArray &)),
 	sink,   SLOT(input(Kwave::SampleArray &)));
-    Q_ASSERT(ok);
-    if (!ok) {
-	close();
-	return;
-    }
 
     // transport the samples
     qDebug("VolumePlugin: filter started...");
